use std::fmt::{Display, Formatter};
use crate::input::events::keyboard::KvmKeyboardEvent;
use crate::input::events::pointer::KvmPointerEvent;

pub mod pointer;
pub mod keyboard;

pub enum KvmEvent {
    Pointer(KvmPointerEvent),
    Keyboard(KvmKeyboardEvent),
}

pub enum KvmButtonState {
    Pressed,
    Released,
}

pub trait KvmEventTrait {
    fn into_kvm_event(self) -> KvmEvent;
}

impl Display for KvmEvent {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        match self {
            KvmEvent::Pointer(_) => write!(f, "Pointer"),
            KvmEvent::Keyboard(_) => write!(f, "Keyboard")
        }
    }
}

impl Display for KvmButtonState {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        match self {
            KvmButtonState::Pressed => write!(f, "Pressed"),
            KvmButtonState::Released => write!(f, "Released")
        }
    }
}