use crate::input::events::{KvmButtonState, KvmEvent, KvmEventTrait};

pub enum KvmPointerEvent {
    Motion(KvmPointerMotionEvent),
    Button(KvmPointerButtonEvent),
    Scroll(KvmPointerScrollEvent),
}

pub struct KvmPointerButtonEvent {
    pub button: u32,
    pub state: KvmButtonState,
}

pub struct KvmPointerMotionEvent {
    pub dx: f64,
    pub dx_unaccelerated: f64,
    pub dy: f64,
    pub dy_unaccelerated: f64,
}

pub struct KvmPointerScrollEvent {
    pub horizontal_scroll_value: f64,
    pub vertical_scroll_value: f64,
}

pub trait KvmPointerEventTrait {
    fn into_kvm_pointer_event(self) -> KvmPointerEvent;
}

impl KvmEventTrait for KvmPointerEvent {
    fn into_kvm_event(self) -> KvmEvent {
        KvmEvent::Pointer(self)
    }
}

macro_rules! kvm_event_traits {
    ($e:ident, $s:ident) => {
        impl KvmEventTrait for $s {
            fn into_kvm_event(self) -> KvmEvent {
                self.into_kvm_pointer_event().into_kvm_event()
            }
        }

        impl KvmPointerEventTrait for $s {
            fn into_kvm_pointer_event(self) -> KvmPointerEvent {
                KvmPointerEvent::$e(self)
            }
        }
    }
}

kvm_event_traits!(Motion, KvmPointerMotionEvent);
kvm_event_traits!(Button, KvmPointerButtonEvent);
kvm_event_traits!(Scroll, KvmPointerScrollEvent);